#ifndef CPGL_H
#define CPGL_H

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <SDL.h>
#ifdef __APPLE__
    #include <OpenGL/glu.h>
    #include <OpenGL/glext.h>
#else
    #include <GL/glu.h>
    #include <GL/glext.h>
    #include <GL/glx.h>
    #include <GL/glxext.h>
#endif

//-----------------------------------
// Function prototypes
char* GetShaderSource(const char* filename);
void UnloadShaders(GLhandleARB& vs, GLhandleARB& fs, GLhandleARB& prg);
void SetupShaders(const char* vert,const char* frag,GLhandleARB& vs,
                  GLhandleARB& fs, GLhandleARB& prg);
void DrawScene();
void ProcessKeyDown(SDL_Event* e);
void ProcessKeyUp(SDL_Event* e);
void ProcessMouseMotion(SDL_Event* e);
void InfLoop();
void Update(double dt);
void InitLight();
void SetupSDL();
void SetupGLProcs();
void CheckGLErrors();
void PrintFBOStatus();

#define FRAME_SRC  0
#define TEXBUF_SRC 1

extern "C" {
    // default texsrc is false so read by glReadPixels
    extern void snap(char* filename, int w, int h, int texsrc);
}

//--------------
#if !defined(__APPLE__) && !defined(_WIN32)

extern PFNGLCREATEPROGRAMOBJECTARBPROC     glCreateProgramObjectARB;
extern PFNGLCREATESHADEROBJECTARBPROC      glCreateShaderObjectARB;
extern PFNGLSHADERSOURCEARBPROC            glShaderSourceARB;
extern PFNGLCOMPILESHADERARBPROC           glCompileShaderARB;
extern PFNGLGETOBJECTPARAMETERIVARBPROC    glGetObjectParameterivARB;
extern PFNGLATTACHOBJECTARBPROC            glAttachObjectARB;
extern PFNGLGETINFOLOGARBPROC              glGetInfoLogARB;
extern PFNGLLINKPROGRAMARBPROC             glLinkProgramARB;
extern PFNGLUSEPROGRAMOBJECTARBPROC        glUseProgramObjectARB;
extern PFNGLGETUNIFORMLOCATIONARBPROC      glGetUniformLocationARB;

extern PFNGLUNIFORM1FARBPROC               glUniform1f;
extern PFNGLUNIFORM2FARBPROC               glUniform2f;
extern PFNGLUNIFORM3FARBPROC               glUniform3f;

extern PFNGLUNIFORM1FVARBPROC              glUniform1fv;
extern PFNGLUNIFORM2FVARBPROC              glUniform2fv;
extern PFNGLUNIFORM3FVARBPROC              glUniform3fv;

extern PFNGLUNIFORM1IARBPROC               glUniform1i;
extern PFNGLDELETEOBJECTARBPROC            glDeleteObjectARB;
extern PFNGLBINDATTRIBLOCATIONARBPROC      glBindAttribLocationARB;
extern PFNGLGETATTRIBLOCATIONARBPROC       glGetAttribLocationARB;
extern PFNGLVERTEXATTRIB3FARBPROC          glVertexAttrib3fARB;

// FBO functions
extern PFNGLGENFRAMEBUFFERSEXTPROC         glGenFramebuffersEXT;
extern PFNGLBINDFRAMEBUFFEREXTPROC         glBindFramebufferEXT;
extern PFNGLDELETEFRAMEBUFFERSEXTPROC      glDeleteFramebuffersEXT;
extern PFNGLFRAMEBUFFERTEXTURE2DEXTPROC    glFramebufferTexture2DEXT;
extern PFNGLCHECKFRAMEBUFFERSTATUSEXTPROC  glCheckFramebufferStatusEXT;

// MULTITEXTURE FUNCTIONS
extern PFNGLACTIVETEXTUREPROC              glActiveTexture;


#define glGetProcAddress(x) (*glXGetProcAddressARB)((const GLubyte*) x)

#endif


#define DESIRED_FPS 60.0

#endif

