#ifndef MODEL_H
#define MODEL_H

#include "cpgl.h"

typedef struct {
    int         obj;
    GLhandleARB vs;
    GLhandleARB fs;
    GLhandleARB prg;
    GLfloat     m[16]; // gl matrix style... column, row order to orient
    GLfloat     pos[3];
    GLfloat     scale[3];
} ObjModel;

// -- model management
//  model file, vertex shader file, fragment shader file
int  load_model      (const char* m, const char* v, const char* f);
int  load_model      (const char* m, int shader_copy);
void free_model      (int i);
void free_all_models ();

// -- model manipulation
// -- absolute manipulations
void position_model  (int i, float x, float y, float z);
void orient_model    (int i, float p, float y, float r);
void scale_model     (int i, float x, float y, float z);
void scale_model     (int i, float s);

// -- relative manipulations
void turn_model      (int i, float p, float y, float r); // in radians
void translate_model (int i, float x, float y, float z);
void stretch_model   (int i, float x, float y, float z);
void stretch_model   (int i, float s);
 
// -- model output
void draw_model(int i, bool shader = true, bool axes = false);

// -- glsl uniform access
GLint get_model_uniform_loc(int i, const char* label);
void  bind_model_shader_prg(int i);
void  set_models_do_lighting(bool b);

#endif
