uniform sampler2D ReflectTex;
uniform sampler2D RefractTex;
uniform sampler2D NormalMapTex;
uniform sampler2D DuDvTex;
uniform sampler2D DepthTex;
uniform int       UnderWater;

varying vec3      evec;
varying vec4      etspace;
varying vec4      ltspace;
varying vec4      viewCoord;

void main()
{
    const float kShine = 128.0;
    const float kDistortion = 0.020;
    const float kRefraction = 0.09;


    vec4 distOffset = texture2D( DuDvTex, gl_TexCoord[0].st) * kDistortion;
    // grab normal map
    vec4 nmap = texture2D(NormalMapTex, 
                                 vec2(gl_TexCoord[0] + distOffset) );
    nmap.a = 0.0;
    vec4  ndelta     = (nmap * 2.0) - 1.0;

    float diffuse    = max(0.0, dot(ndelta,normalize(ltspace)));
    vec3  light_refl = normalize( reflect( -ltspace.xyz, ndelta.xyz) );
    float invFresnel = .5 * dot( ndelta, -etspace );
    float Fresnel    = 1.0 - invFresnel;

    vec4  reflectclr;
    vec4  refractclr;
    vec4  depthval;
    vec4  reflectoffset;
    float deltadepth;
    vec4 dudvclr;
    dudvclr  = texture2D( DuDvTex, vec2( gl_TexCoord[0] + distOffset));
    dudvclr  = normalize( dudvclr * 2.0 - 1.0);
    dudvclr *= kRefraction;

    vec4 dudvalt_rfr = vec4(normalize(
        refract( etspace.xyz, normalize(ndelta.xyz), .66)), 1.0);
    vec4 dudvalt_rfl = vec4(normalize(
        reflect( etspace.xyz, normalize(ndelta.xyz))), 1.0);
   
    vec4 final_tc;
    vec4 final_tc_rfr;
    vec4 final_tc_rfl;

    final_tc = viewCoord / viewCoord.q;
    final_tc = (final_tc + 1.0) * .5;
    final_tc_rfr = final_tc + dudvalt_rfr;
    final_tc_rfl = final_tc + dudvalt_rfl;
    final_tc += dudvclr;
    final_tc = clamp( final_tc, 0.001, 0.999);
    final_tc_rfr = clamp( final_tc_rfr, 0.001, 0.999);
    final_tc_rfl = clamp( final_tc_rfl, 0.001, 0.999);
   
    final_tc_rfr = final_tc;
    final_tc_rfl = final_tc;
    // bend based on normal 
    
    reflectclr  = texture2D( ReflectTex, final_tc_rfl.st);
    refractclr  = texture2D( RefractTex, final_tc_rfr.st);
    depthval    = texture2D( DepthTex,   final_tc.st);
    
    vec4 invDepth = 1.0 - depthval;
    if( UnderWater == 0)
    {
        refractclr *= invFresnel;
        refractclr += gl_FrontMaterial.diffuse * depthval * invFresnel;
        reflectclr *= Fresnel;
    }

    vec4 localview  = normalize(etspace);
    float intensity = max(0.0, dot( light_refl, localview.xyz));
    float specular  = pow( intensity, kShine );
    if( UnderWater == 0 )
        reflectclr = vec4( vec3(reflectclr + refractclr), 1.0);
    else
        reflectclr = vec4( vec3(reflectclr * .2 + refractclr), 1.0);

    gl_FragColor = reflectclr;
}

