uniform sampler2D RM; // Relief Map
uniform sampler2D CM; // Color Map


float depth = .2;

varying vec3 light;
varying vec3 view;

vec3 search(vec3 A, vec3 B)
{
    vec3 D = (B - A) / 32.0;
    vec3 I;

    // The linear search.

    B = A;

    for (int i = 0; i < 32; ++i)
    {
        if (texture2D(RM, B.xy).a > B.z)
            B = B + D;
        else
            break;
    }

    // The binary search.

    A = B - D;

    for (int i = 0; i < 8; ++i)
    {
        I = (A + B) / 2.0;

        if (texture2D(RM, I.xy).a > I.z)
            A = I;
        else
            B = I;
    }

    return (A + B) / 2.0;
}

void main()
{
    vec3 L = normalize(light);
    vec3 V = normalize(view);
    vec3 H = normalize(L - V);

    // Find the view ray's entry into and exit from the surface bounding box.

    vec3 A = vec3(gl_TexCoord[0].xy,                      0.0);
    vec3 B = vec3(gl_TexCoord[0].xy - depth * V.xy / V.z, 1.0);

    // Find the view ray height map intersection.

    vec3 I = search(A, B);

    // Find the light ray's entry into the surface bounding box.

    vec3 C = vec3(I.xy + depth * L.xy * I.z / L.z, 0.0);

    // Find the light ray height map intersection.

    vec3  J = search(C, I);
    float k = step(0.99, 1.0 - (I.z - J.z));

    // We know where we're at in the texture.  Color and light the pixel.

    vec4 Ka = gl_LightModel.ambient;
    vec4 Kd =           texture2D(RM,  I.xy);
    vec3 N  = normalize(texture2D(RM, I.xy).xyz*2.0-1.0);

    float d =     max(0.0, dot(L, N));
    float s = pow(max(0.0, dot(H, N)), 32.0);

    gl_FragColor = vec4(((Kd.rgb * d) + s) * k + Kd.rgb * Ka.rgb, Kd.a * Ka.a);
//  gl_FragColor = vec4(vec3(k), 1.0);
}

