#include "camera.h"
#include "matrix.h"
#include <vector>
#include <math.h>

//------------------------------------
Camera gCamera;

//------------------------------------
void position_camera( float x, float y, float z)
{
    gCamera.pos[0] = x;
    gCamera.pos[1] = y;
    gCamera.pos[2] = z;
}

//------------------------------------
void orient_camera( float p, float y, float r)
{
    GLfloat *m = gCamera.m;
    GLfloat a[16];
    GLfloat b[16];
    make_identity(m);
    make_identity(a);
    make_identity(b);

    GLfloat cr = cosf(r);
    GLfloat sr = sinf(r);
    GLfloat cp = cosf(p);
    GLfloat sp = sinf(p);
    GLfloat cy = cosf(y);
    GLfloat sy = sinf(y);

    // yaw then pitch then roll
    b[5]  = cp;
    b[6]  = sp;
    b[9]  = -sp;
    b[10] = cp;
    a[0]  = cy;
    a[2]  = -sy;
    a[8]  = sy;
    a[10] = cy;

    mul_matrix(m,a,b);
    copy_matrix(a,m);
    make_identity(b);

    b[0] = cr;
    b[1] = sr;
    b[4] = -sr;
    b[5] = cr;

    mul_matrix(m,a,b);
}

//------------------------------------
void orient_camera(float* mat)
{
    if( !mat ) return;
    
    for(int i = 0; i < 16; i++)
        gCamera.m[i] = mat[i];
}

//------------------------------------
void get_camera_position(float *x, float *y, float *z)
{
    if( !x || !y || !z ) return;
    
    *x = gCamera.pos[0];
    *y = gCamera.pos[1];
    *z = gCamera.pos[2];
}

//------------------------------------
void get_camera_orientation(float* r)
{
    if(!r) return;
    for(int i = 0; i < 16; i++)
        r[i] = gCamera.m[i];
}

//------------------------------------
void translate_camera( float x, float y, float z)
{
    // orient vector based on our rotation matrix
    float tx, ty, tz;
    GLfloat *m = gCamera.m;
    tx = (m[0] * x) + (m[4] * y) + (m[8] * z);
    ty = (m[1] * x) + (m[5] * y) + (m[9] * z);
    tz = (m[2] * x) + (m[6] * y) + (m[10] * z);

    // add our new vector to our position vector
    gCamera.pos[0] += tx;
    gCamera.pos[1] += ty;
    gCamera.pos[2] += tz;
}

//------------------------------------
void turn_camera( float p, float y, float r)
{
    GLfloat *m = gCamera.m;
    GLfloat a[16];
    GLfloat b[16];
 
    make_identity(b);
    make_identity(a);

    GLfloat cr = cosf(r);
    GLfloat sr = sinf(r);
    GLfloat cp = cosf(p);
    GLfloat sp = sinf(p);
    GLfloat cy = cosf(y);
    GLfloat sy = sinf(y);

    // yaw then pitch then roll
    GLfloat temp[16];

    a[0]  = cy;
    a[2]  = -sy;
    a[8]  = sy;
    a[10] = cy;
    
    b[5]  = cp;
    b[6]  = sp;
    b[9]  = -sp;
    b[10] = cp;
    
    mul_matrix(temp,a,b);
    copy_matrix(a,temp);
    make_identity(b);
    
    b[0] = cr;
    b[1] = sr;
    b[4] = -sr;
    b[5] = cr;
    
    mul_matrix(temp,a,b);
    copy_matrix(a,m);
    mul_matrix(m,a,temp);

}

// make sure by accident it these can't happen more than once
void apply_camera_matrix()
{
    gluLookAt( gCamera.pos[0], gCamera.pos[1], gCamera.pos[2],
               gCamera.pos[0] + gCamera.m[8], 
               gCamera.pos[1] + gCamera.m[9], 
               gCamera.pos[2] + gCamera.m[10],
               gCamera.m[4], gCamera.m[5], gCamera.m[6]);

}

void unapply_camera_matrix()
{
}

