#include "cpgl.h"
#include "trimodel.h"
#include <math.h>

TriModel* LoadTriModel(const char* filename)
{
    if( !filename ) return NULL;
    
    FILE* fptr = fopen(filename,"r");    
    
    if(!fptr) return NULL;
    
    int nfaces;
    
    fscanf(fptr,"%d\n",&nfaces);

    if( !nfaces ) {
        fclose(fptr);
        return NULL;
    }
    
    TriModel* tm = new TriModel();
    tm->groups = new TriGroup[1];
    tm->ngroups = 1;
    
    TriGroup* tg = &(tm->groups[0]);
    tg->nelements = nfaces * 3;
    tg->verts = new GLfloat[ tg->nelements * 3];
    tg->norms = new GLfloat[ tg->nelements * 3];
    
    for( int i = 0; i < tg->nelements; i++)
    {
        fscanf(fptr,"%f %f %f %f %f %f\n",
            &(tg->verts[ 3 * i]),
            &(tg->verts[ 3 * i + 1]),
            &(tg->verts[ 3 * i + 2]),
            &(tg->norms[ 3 * i]),
            &(tg->norms[ 3 * i + 1]),
            &(tg->norms[ 3 * i + 2])
            );
    
        // normalize normals... just in case;
        float mag = sqrtf( tg->norms[ 3 * i ] * tg->norms[ 3 * i ] +
                            tg->norms[ 3 * i + 1] * tg->norms[ 3 * i + 1] +
                            tg->norms[ 3 * i + 2] * tg->norms[ 3 * i + 2]);
        tg->norms[ 3 * i ] /= mag;
        tg->norms[ 3 * i  + 1] /= mag;
        tg->norms[ 3 * i  + 2] /= mag;
    }

    fclose(fptr);
    tm->vs = 0;
    tm->fs = 0;
    tm->prg = 0;
    return tm;
}


void FreeTriModel(TriModel* t)
{
    if( !t ) return;

    for( int i = 0; i < t->ngroups; i++)
    {
        if( t->groups[i].verts)
            delete [] t->groups[i].verts;
        if( t->groups[i].norms)
            delete [] t->groups[i].norms;

    }

    delete [] t->groups;
    t->ngroups = 0;
}

void DrawTriModel(TriModel* t)
{
    if( !t ) return;
    
    glUseProgramObjectARB(t->prg);

    for( int i = 0; i < t->ngroups; i++)
    {
        glVertexPointer(3,GL_FLOAT,0,t->groups[i].verts);
        glNormalPointer(GL_FLOAT,0,t->groups[i].norms);
        glDrawArrays(GL_TRIANGLES,0,t->groups[i].nelements);
    }
}
